/*
 *  libpipi       Proper image processing implementation library
 *  Copyright (c) 2004-2008 Sam Hocevar <sam@zoy.org>
 *                All Rights Reserved
 *
 *  $Id$
 *
 *  This library is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

/*
 * image.c: image I/O functions
 */

#include "config.h"
#include "common.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <cv.h>
#include <highgui.h>

#include "pipi.h"
#include "pipi_internals.h"

/* FIXME: this whole file is broken until we support BGR24 images */

pipi_image_t *pipi_load_opencv(const char *name)
{
    pipi_image_t *img;
    IplImage *priv = cvLoadImage(name, -1);

    if(!priv)
        return NULL;

    img = (pipi_image_t *)malloc(sizeof(pipi_image_t));
    memset(img, 0, sizeof(pipi_image_t));

    img->w = priv->width;
    img->h = priv->height;

    img->p[PIPI_PIXELS_RGBA32].pixels = priv->imageData;
    img->p[PIPI_PIXELS_RGBA32].w = priv->width;
    img->p[PIPI_PIXELS_RGBA32].h = priv->height;
    img->p[PIPI_PIXELS_RGBA32].pitch = priv->widthStep;
    img->last_modified = PIPI_PIXELS_RGBA32;

    img->codec_priv = (void *)priv;
    img->codec_format = PIPI_PIXELS_RGBA32;

    return img;
}

pipi_image_t *pipi_new_opencv(int width, int height)
{
    pipi_image_t *img;
    IplImage *priv = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 4);

    if(!priv)
        return NULL;

    img = (pipi_image_t *)malloc(sizeof(pipi_image_t));
    memset(img, 0, sizeof(pipi_image_t));

    img->w = priv->width;
    img->h = priv->height;

    img->p[PIPI_PIXELS_RGBA32].pixels = priv->imageData;
    img->p[PIPI_PIXELS_RGBA32].w = priv->width;
    img->p[PIPI_PIXELS_RGBA32].h = priv->height;
    img->p[PIPI_PIXELS_RGBA32].pitch = priv->widthStep;
    img->last_modified = PIPI_PIXELS_RGBA32;

    img->codec_priv = (void *)priv;
    img->codec_format = PIPI_PIXELS_RGBA32;

    return img;
}

void pipi_free_opencv(pipi_image_t *img)
{
    IplImage *iplimg;
    iplimg = (IplImage *)img->codec_priv;
    cvReleaseImage(&iplimg);

    free(img);
}

void pipi_save_opencv(pipi_image_t *img, const char *name)
{
    pipi_getpixels(img, img->codec_format);
    cvSaveImage(name, img->codec_priv);
}

